#include "MeshLoader.h"
#include <algorithm>


std::vector<Node>& MeshLoader::GetNode() {
	return node;
}

std::vector<FiniteElement>& MeshLoader::GetFiniteElement() {
	return finite_el;
}

std::vector<FiniteElement>& MeshLoader::GetBoundaryElement() {
	return boundary_el;
}

std::vector<int> MeshLoader::ElementByThreeNodes(int id1, int id2, int id3) {
	std::vector<int> result;
	auto a = finite_el.begin();
	while (a != finite_el.end()) {
		a = find_if(a, finite_el.end(), [id1, id2, id3](FiniteElement s) { //!!! Копирование s на каждой итерации
		    
			return (find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id1) != s.ID_Nodes.end() &&
				find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id2) != s.ID_Nodes.end() &&
				find(s.ID_Nodes.begin(), s.ID_Nodes.end(), id3) != s.ID_Nodes.end());
			});
		if (a != finite_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}


std::vector<int> MeshLoader::ElementByTwoNodes(int id1, int id2) {
	std::vector<int> result;
	auto a = finite_el.begin();
	while (a != finite_el.end()) {
		a = find_if(a, finite_el.end(), [id1, id2](const FiniteElement& elem) {
			return (find(elem.ID_Nodes.begin(), elem.ID_Nodes.end(), id1) != elem.ID_Nodes.end() &&
				find(elem.ID_Nodes.begin(), elem.ID_Nodes.end(), id2) != elem.ID_Nodes.end());
			});
		if (a != finite_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}

std::vector<int> MeshLoader::FiniteElementsByAreaID(int id) {
	std::vector<int> result;
	auto a = finite_el.begin();
	while (a!= finite_el.end()) {
			a = find_if(a, finite_el.end(),
			[id](const FiniteElement& elem) {
				return elem.ID_Material == id;
			});
		if (a != finite_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}


void MeshLoader::insert_middle() {
	std::unordered_set<Edge, EdgeHash> List_edge;
	
	for (auto& i : finite_el) {
		std::vector<int> a = i.ID_Nodes;
		for (auto elem1 = a.begin(); elem1 < a.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < a.end(); elem2++) {
				Edge ed(*elem1, *elem2);
				auto el = List_edge.find(ed);
				if (el == List_edge.end()) {
					Node el;
					el.ID_Node = node.size() + 1;
					el.flag = false;
					Node center_node(el.ID_Node,
						std::array<double, 3>{((node.at(*elem1 - 1).Coordinate.at(0) + node.at(*elem2 - 1).Coordinate.at(0)) / 2),
						((node.at(*elem1 - 1).Coordinate.at(1) + node.at(*elem2 - 1).Coordinate.at(1)) / 2),
						((node.at(*elem1 - 1).Coordinate.at(2) + node.at(*elem2 - 1).Coordinate.at(2)) / 2)}, false);
					node.push_back(center_node);
					i.ID_Nodes.push_back(el.ID_Node);
					ed.middle_id = el.ID_Node;
					List_edge.insert(ed);
				}
				else {
					i.ID_Nodes.push_back(el->middle_id);
				}
			}
		}
	}
	for (auto& element : boundary_el) { 
		std::vector<int> a = element.ID_Nodes;
		for (auto elem1 = a.begin(); elem1 < a.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < a.end(); elem2++) {
				Edge ed(*elem1, *elem2);
				auto el = List_edge.find(ed);
				element.ID_Nodes.push_back(
					el->middle_id); 
			}
		}
	}
}


std::vector<std::set<int>> MeshLoader::Neighbor_Nodes() {
	std::vector<std::set<int>> result(node.size());
	for (auto& elem : finite_el) {
		for (auto elem1 = elem.ID_Nodes.begin(); elem1 < elem.ID_Nodes.end(); elem1++) {
			for (auto elem2 = elem1 + 1; elem2 < elem.ID_Nodes.end(); elem2++) {
				result[*elem1 - 1].insert(*elem2);
			}
		}
	}
	return result;
}


void MeshLoader::print_DataTypes() {
	std::cout << "Nodes:" << std::endl << node << std::endl
		<< "Finite Elements:" << std::endl << finite_el << std::endl;
}

std::set<int> MeshLoader::BoundaryNodesByBoundaryID(int id) {
	std::set<int> result;
	auto a = boundary_el.begin();
	while (a != boundary_el.end()) {
		a = find_if(a, boundary_el.end(),
			[id](const FiniteElement& elem) {
				return elem.ID_Material == id;
			});
		if (a != boundary_el.end()) {
			for (auto element : a->ID_Nodes) {
				result.insert(element);
			}
		}
		++a;
	}
	return result;
}

std::vector<int> MeshLoader::BoundaryElementsByBoundaryID(int id) {
	std::vector<int> result;
	auto a = boundary_el.begin();
	while (a != boundary_el.end()) {
		a = find_if(a, boundary_el.end(),
			[id](const FiniteElement& elem) {
				return elem.ID_Material == id;
			});
		if (a != boundary_el.end()) {
			result.push_back(a->ID_FiniteElement);
			a++;
		}
	}
	return result;
}

